package com.mortals.xhx.module.menu.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.common.code.YesNo;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.IUser;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.pdu.ModelPdu;
import com.mortals.xhx.feign.model.IApiModelFeign;
import com.mortals.xhx.feign.model.vo.ModelCensusFeignVO;
import com.mortals.xhx.feign.model.vo.ModelFeignVO;
import com.mortals.xhx.feign.rsp.ApiResp;
import com.mortals.xhx.module.menu.model.MenuQuery;
import com.mortals.xhx.module.menu.model.vo.MenuNodeVO;
import com.mortals.xhx.module.role.service.RoleModelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.menu.dao.MenuDao;
import com.mortals.xhx.module.menu.model.MenuEntity;
import com.mortals.xhx.module.menu.service.MenuService;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * MenuService
 * 菜单信息业务 service实现
 *
 * @author zxfei
 * @date 2022-05-25
 */
@Service("menuService")
public class MenuServiceImpl extends AbstractCRUDServiceImpl<MenuDao, MenuEntity, Long> implements MenuService {

    @Autowired
    private IApiModelFeign apiModelFeign;
    @Autowired
    private RoleModelService roleModelService;

    @Override
    public int remove(Long[] ids, Context context) throws AppException {
        //删除后级联删除子节点
        Arrays.asList(ids).stream().forEach(id->{
            menudelete(context, id);
        });
        return super.remove(ids, context);
    }

    private void menudelete(Context context, Long id) {
        Long[] ids = this.find(new MenuQuery().parentId(id)).stream().map(item -> item.getId()).toArray(Long[]::new);
        if(!ObjectUtils.isEmpty(ids)){
            this.remove(ids, context);
            Arrays.asList(ids).forEach(item->{
                menudelete(context,item);
            });
        }
    }

    @Override
    protected void findAfter(MenuEntity params, PageInfo pageInfo, Context context, List<MenuEntity> list) throws AppException {
        //查询当前目录等级
        list.stream().forEach(item -> {
            HashMap<Integer, MenuEntity> map = new HashMap<>();
            int count = 1;
            buildMenuMap(context, item, map, count);
            if (map.size() == 0) {
                //当前菜单为顶级
                item.setFirstLevel(item.getName());
                item.setFirstMenuId(item.getId());
                item.setSecondLevel("");
                item.setThirdLevel("");
            } else if (map.size() == 1) {
                item.setFirstLevel(map.get(1).getName());
                item.setFirstMenuId(map.get(1).getId());
                item.setSecondLevel(item.getName());
                item.setSecondMenuId(item.getId());
                item.setThirdLevel("");
            } else if (map.size() == 2) {
                item.setFirstLevel(map.get(2).getName());
                item.setFirstMenuId(map.get(2).getId());
                item.setSecondLevel(map.get(1).getName());
                item.setSecondMenuId(map.get(1).getId());
                item.setThirdLevel(item.getName());
                item.setThirdMenuId(item.getId());
            } else {

            }
        });
        super.findAfter(params, pageInfo, context, list);
    }

    private Map<Integer, MenuEntity> buildMenuMap(Context context, MenuEntity item, Map<Integer, MenuEntity> map, int count) {
        if (!ObjectUtils.isEmpty(item.getParentId())) {
            MenuEntity menuEntity = this.get(item.getParentId(), context);
            if (!ObjectUtils.isEmpty(menuEntity)) {
                map.put(count, menuEntity);
                count++;
                return buildMenuMap(context, menuEntity, map, count);
            }
        }
        return map;
    }

    @Override
    public List<MenuEntity> findAllEnable() throws AppException {
        MenuQuery query = new MenuQuery();
        query.setStatus(YesNo.YES.getValue());
        List<OrderCol> orderColList = new ArrayList<OrderCol>();
        orderColList.add(new OrderCol("parentId"));
        orderColList.add(new OrderCol("orderId"));
        query.setOrderColList(orderColList);
        List<MenuEntity> menuEntityList = this.find(query);
        List<MenuEntity> menuTreeList = getChildPerms(menuEntityList, -1);

        return menuTreeList;
    }

    @Override
    public List<MenuEntity> findTreeMenuByUser(IUser user) throws AppException {
        if (!user.isAdmin()) {
            List<MenuEntity> menuList = dao.selectMenuTreeByUserId(user.getId());
            //转换为树结构
            List<MenuEntity> menuTreeList = getChildPerms(menuList, -1);
            return menuTreeList;
        } else {
            return findAllEnable();
        }
    }


    @Override
    public void upOrDown(Long id, Integer type) {
        MenuQuery query = new MenuQuery();
        query.setOrderColList(Arrays.asList(new OrderCol("orderId")));
        query.setOrderKind(OrderCol.ASCENDING);
        //判断移动的是一级菜单还是二级菜单
        MenuEntity menuEntity = this.get(id);
        if (menuEntity.getParentId() == 0) {
            //一级菜单
            query.setParentId(0L);
        } else {
            //二级菜单
            query.setParentId(menuEntity.getParentId());
        }
        List<MenuEntity> list = this.find(query);
        MenuEntity preEntity = null;
        MenuEntity curEntity = null;
        if (type == YesNoEnum.NO.getValue()) {
            //up
            for (int i = 0; i < list.size(); i++) {
                if (list.get(i).getId().equals(id) && i > 0) {
                    curEntity = list.get(i);
                    preEntity = list.get(i - 1);
                    break;
                }
            }

        } else {
            //down
            for (int i = 0; i < list.size(); i++) {
                if (list.get(i).getId().equals(id) && i + 1 < list.size()) {
                    curEntity = list.get(i);
                    preEntity = list.get(i + 1);
                    break;
                }
            }
        }

        if (preEntity != null && curEntity != null) {
            //交换
            Integer temp = preEntity.getOrderId();
            preEntity.setOrderId(curEntity.getOrderId());
            curEntity.setOrderId(temp);

            this.update(preEntity);
            this.update(curEntity);
        }

    }

    @Override
    protected void saveBefore(MenuEntity entity, Context context) throws AppException {
        MenuQuery query = new MenuQuery();
        query.setParentId(entity.getParentId());
        Comparator<Integer> comparator = Comparator.comparing(Integer::intValue);
        Optional<Integer> maxOptional = this.find(query).stream().map(MenuEntity::getOrderId).filter(f -> f != 9999).max(comparator);
        maxOptional.ifPresent(e -> {
            e = e + 1;
            entity.setOrderId(e);
        });
        super.saveBefore(entity, context);
    }


    /**
     * 根据父节点的ID获取所有子节点
     *
     * @param list     分类表
     * @param parentId 传入的父节点ID
     * @return String
     */
    public List<MenuEntity> getChildPerms(List<MenuEntity> list, int parentId) {
        List<MenuEntity> returnList = new ArrayList<MenuEntity>();
        for (Iterator<MenuEntity> iterator = list.iterator(); iterator.hasNext(); ) {
            MenuEntity t = iterator.next();
            // 一、根据传入的某个父节点ID,遍历该父节点的所有子节点
            if (t.getParentId() == parentId) {
                recursionFn(list, t);
                returnList.add(t);
            }
        }
        return returnList;
    }

    /**
     * 递归列表
     *
     * @param list
     * @param t
     */
    private void recursionFn(List<MenuEntity> list, MenuEntity t) {
        // 得到子节点列表
        List<MenuEntity> childList = getChildList(list, t);
        t.setChildList(childList);
        for (MenuEntity tChild : childList) {
            if (hasChild(list, tChild)) {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<MenuEntity> getChildList(List<MenuEntity> list, MenuEntity t) {
        List<MenuEntity> tlist = new ArrayList<>();
        Iterator<MenuEntity> it = list.iterator();
        while (it.hasNext()) {
            MenuEntity n = it.next();
            if (n.getParentId().longValue() == t.getId().longValue()) {
                tlist.add(n);
            }
        }
        return tlist;
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<MenuEntity> list, MenuEntity t) {
        return getChildList(list, t).size() > 0 ? true : false;
    }


    @Override
    public List<MenuNodeVO> getMenuList() throws AppException {
        MenuQuery query = new MenuQuery();
        query.setStatus(YesNo.YES.getValue());
        List<OrderCol> orderColList = new ArrayList<OrderCol>();
        orderColList.add(new OrderCol("parentId"));
        orderColList.add(new OrderCol("orderId"));
        query.setOrderColList(orderColList);
        List<MenuEntity> menuEntityList = this.find(query);
        List<MenuNodeVO> nodeList = new ArrayList<>();
        for (MenuEntity menuEntity:menuEntityList){
            MenuNodeVO nodeVO = new MenuNodeVO(menuEntity.getName(),menuEntity.getId(),menuEntity.getParentId(),3);
            nodeList.add(nodeVO);
        }
        List<MenuNodeVO> tree = creatMenuNodeTree(nodeList);
        tree.forEach(item->{
            if(item.getNodeName().equals("数据管理")){
                item.setChildList(creatCensus(item.getNodeId()));
            }
            if(item.getNodeName().equals("站点编排")){
                item.setChildList(creatModel(item.getNodeId()));
            }
        });

        return tree;
    }

    private List<MenuNodeVO> creatMenuNodeTree(List<MenuNodeVO> list){
        Map<Long, List<MenuNodeVO>> sub = list.stream().filter(node -> node.getParentId() != -1).collect(Collectors.groupingBy(node -> node.getParentId()));
        list.forEach(node -> node.setChildList(sub.get(node.getNodeId())));
        return list.stream().filter(node -> node.getParentId() == -1).collect(Collectors.toList());
    }

    private List<MenuNodeVO> creatModel(Long nodeId){
        ModelPdu modelPdu = new ModelPdu();
        //ApiResp<List<ModelFeignVO>> apiResp = apiModelFeign.getModelByQuery(modelPdu);
        String resp = apiModelFeign.getModelByQuery(modelPdu);
        ApiResp<JSONObject> apiResp = JSON.parseObject(resp, ApiResp.class);
        if (apiResp.getCode() != YesNoEnum.YES.getValue()) {
            throw new AppException("获取站点列表信息失败：" + apiResp.getMsg());
        }
        List<ModelFeignVO> modelFeignVOS = JSONObject.parseArray(apiResp.getData().get("data").toString(),ModelFeignVO.class);
        List<MenuNodeVO> nodeList = new ArrayList<>();
        modelFeignVOS.stream().forEach(item->{
            MenuNodeVO nodeVO = new MenuNodeVO(item.getModelName(),item.getId(),nodeId,2);
            nodeList.add(nodeVO);
        });
        return nodeList;
    }

    private List<MenuNodeVO> creatCensus(Long nodeId){
        ModelPdu modelPdu = new ModelPdu();
        //ApiResp<List<ModelCensusFeignVO>> apiResp = apiModelFeign.getModelCensusByQuery(modelPdu);
        String resp = apiModelFeign.getModelCensusByQuery(modelPdu);
        ApiResp<JSONObject> apiResp = JSON.parseObject(resp, ApiResp.class);
        if (apiResp.getCode() != YesNoEnum.YES.getValue()) {
            throw new AppException("获取所有数据管理列表信息失败：" + apiResp.getMsg());
        }
        List<ModelCensusFeignVO> modelCensusFeignVOS = JSONObject.parseArray(apiResp.getData().get("data").toString(),ModelCensusFeignVO.class);
        List<MenuNodeVO> nodeList = new ArrayList<>();
        modelCensusFeignVOS.stream().forEach(item->{
            MenuNodeVO nodeVO = new MenuNodeVO(item.getCensusName(),item.getId(),nodeId,1);
            nodeList.add(nodeVO);
        });
        return nodeList;
    }

}