package com.mortals.xhx.module.menu.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.common.code.*;
import com.mortals.xhx.module.menu.model.MenuEntity;
import com.mortals.xhx.module.menu.model.vo.MenuNodeVO;
import com.mortals.xhx.module.menu.service.MenuService;
import com.mortals.xhx.module.param.service.ParamService;
import com.mortals.xhx.module.role.model.RoleAuthEntity;
import com.mortals.xhx.module.role.model.RoleAuthQuery;
import com.mortals.xhx.module.role.service.RoleAuthService;
import com.mortals.xhx.module.role.service.RoleModelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 菜单信息业务
 *
 * @author zxfei
 * @date 2022-05-25
 */
@RestController
@RequestMapping("menu")
public class MenuController extends BaseCRUDJsonBodyMappingController<MenuService, MenuEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private RoleAuthService roleAuthService;
    @Autowired
    private RoleModelService roleModelService;

    public MenuController() {
        super.setModuleDesc("菜单信息业务");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        Map<String, Object> status = new HashMap<String, Object>();
        status.put("status", DataSatus.getEnumMap(DataSatus.CLOSE.getValue(), DataSatus.DELETE.getValue(), DataSatus.OVERDUE.getValue(), DataSatus.USEOUT.getValue()));
        status.put("linkType", MenuLinkType.getEnumMap());
        status.put("commMenu", MenuComm.getEnumMap());
        status.put("menuType", MenuType.getEnumMap());
        status.put("authType", MenuAuthType.getEnumMap());
        model.put(KEY_RESULT_DICT, status);
        super.init(model, context);
    }


    @Override
    protected void doListBefore(MenuEntity query, Map<String, Object> model, Context context) throws AppException {
        List<OrderCol> orderColList = new ArrayList<OrderCol>();
        orderColList.add(new OrderCol("parentId"));
        orderColList.add(new OrderCol("orderId"));
        query.setOrderColList(orderColList);
        super.doListBefore(query, model, context);
    }


    /**
     * 改变状态
     */
    @RequestMapping(value = "change/status")
    public String changeStatus(@RequestParam(value = "id") Long id, @RequestParam(value = "status") Integer status) {
        JSONObject ret = new JSONObject();
        Context context = getContext();
        try {
            MenuEntity entity = this.service.get(id, context);//.doSubmitAudit(form.getEntity(), context);
            if (null == entity) {
                throw new AppException("菜单不存在!");
            }
            if (null == status) {
                throw new AppException("菜单状态不能为空!");
            }
            if (status != DataSatus.ENABLE.getValue() && status != DataSatus.DISENABLE.getValue()) {
                throw new AppException("非法菜单状态!");
            }
            String busiDesc = DataSatus.getByValue(status).getDesc();
            entity.setStatus(status);
            this.service.update(entity, context);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            ret.put(KEY_RESULT_MSG, busiDesc + "成功");
            recordSysLog(request, busiDesc + " 【成功】 [id:" + entity.getId() + "]");

        } catch (Exception e) {
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return ret.toJSONString();
    }


    /**
     * 更换排序
     */
    @PostMapping(value = "upOrDown")
    public String upOrDownTopicList(@RequestParam(value = "id") Long id, @RequestParam(value = "type") Integer type) {
        JSONObject ret = new JSONObject();
        ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
        try {
            this.service.upOrDown(id, type);
        } catch (Exception e) {
            log.error("更新错误", e);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        return ret.toJSONString();
    }

    /**
     * 查询所有可用菜单（树）
     */
    @PostMapping(value = "findAll")
    public String findAll() {
        JSONObject ret = new JSONObject();
        ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
        try {
            List<MenuEntity> menuEntityList = this.service.findAllEnable();
            ret.put(KEY_RESULT_DATA,menuEntityList);
        } catch (Exception e) {
            log.error("更新错误", e);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        return ret.toJSONString();
    }


    /**
     * 查询所有可用菜单（树）
     */
    @PostMapping(value = "list/tree")
    public String getMenuList(@RequestBody RoleAuthEntity roleAuthEntity) {
        JSONObject ret = new JSONObject();
        ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
        try {
            if(roleAuthEntity.getRoleId()==null){
                throw new AppException("角色Id不能为空");
            }
            List<MenuNodeVO> menuEntityList = this.service.getMenuList();
            Map<String,Object> data = new HashMap<>();
            data.put("menu",menuEntityList);
            Map<String,List<Long>> models = roleModelService.getRoleModelByRole(roleAuthEntity.getRoleId());
            data.put("zdbp",models.get("modelIds"));
            data.put("sjgl",models.get("censusIds"));
            Map<Long,List<Long>> menus = roleAuthService.getMenuIdByRole(roleAuthEntity.getRoleId());
            data.put("sjjs",menus.get(3l));
            data.put("ptsz",menus.get(4l));
            ret.put(KEY_RESULT_DATA,data);
        } catch (Exception e) {
            log.error("更新错误", e);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        return ret.toJSONString();
    }


}